local i, keystone,_,bag,bagSize



function KeystoneRollCall:keystonesMatch(k1,k2)
    if k1 == k2 then return true end
    local o1Type = type(k1)
    local o2Type = type(k2)
    if o1Type ~= o2Type then return false end
    if o1Type ~= 'table' then return false end

    local keySet = {}

    for key1, value1 in pairs(k1) do
        local value2 = k2[key1]
        if value2 == nil or self:keystonesMatch(value1, value2) == false then
            return false
        end
        keySet[key1] = true
    end

    for key2, _ in pairs(k2) do
        if not keySet[key2] then return false end
    end
    return true
end

function KeystoneRollCall:purgeExpiredKeystones()
	local playerKeystone = self:getPlayerKeystone()
	if not playerKeystone or not playerKeystone.affix1 or playerKeystone.affix1 <= 0 then
		return 
	end
	
	for i = #self.db.global.keystones, 1, -1 do
		local affix1 = self.db.global.keystones[i].affix1;
		if affix1 and affix1 > 0 and affix1 ~= playerKeystone.affix1 then
			tremove(self.db.global.keystones,i)
		end
	end
end

function KeystoneRollCall:extractKeystone(keystone)
	if not keystone then
		return
	end
	return keystone.name, keystone.class,keystone.faction and 1 or 0,keystone.dungeon,keystone.level,keystone.affix1 or 0,keystone.affix2 or 0,keystone.affix3 or 0,keystone.bestLevel,keystone.bestDungeon
end

function KeystoneRollCall:buildKeystone(args)
	local playerKeystone = self:getPlayerKeystone()
	local keystone = {}	
	keystone.name = args[1]
	keystone.class = args[2]
	keystone.faction = args[3] == "1"
	keystone.dungeon = tonumber(args[4])
	keystone.level = tonumber(args[5])
	keystone.affix1 = tonumber(args[6])
	keystone.affix2 = tonumber(args[7])
	keystone.affix3 = tonumber(args[8])
	keystone.bestLevel = tonumber(args[9])
	keystone.bestDungeon = tonumber(args[10])
	keystone.isPlayers = false;
	
	local oldKeystoneIndex = self:getKeystoneIndex(keystone.name)
	if oldKeystoneIndex and oldKeystoneIndex > 0 then
		local oldKeystone = self.db.global.keystones[oldKeystoneIndex]
		if oldKeystone.isPlayers then
			keystone.isPlayers = true;
		end
	end
	
	return self:saveKeystone(keystone)
end


function KeystoneRollCall:saveKeystone(keystone)
	if not keystone then
		return nil;
	end
	
	local purge = false;
	local playerKeystone = self:getPlayerKeystone()
	if playerKeystone  and playerKeystone.affix1 and playerKeystone.affix1 > 0 and keystone.affix1 and keystone.affix1 > 0 and keystone.affix1 ~= playerKeystone.affix1 then
		if playerKeystone.name == keystone.name then
			purge = true;
		else
			return nil;
		end
	end

	local oldKeystoneIndex = self:getKeystoneIndex(keystone.name);
	if oldKeystoneIndex and oldKeystoneIndex > 0 then
		local changed = false;
		local oldKeystone = self.db.global.keystones[oldKeystoneIndex]
		changed = not KeystoneRollCall:keystonesMatch(oldKeystone,keystone);
		self.db.global.keystones[oldKeystoneIndex] = keystone;
		self:purgeExpiredKeystones()
		return changed and keystone or nil, oldKeystone
	end

	tinsert(self.db.global.keystones,keystone);
	return keystone;
end 

function KeystoneRollCall:buildPlayerKeystone()
	C_ChallengeMode.RequestMapInfo();
	C_ChallengeMode.RequestRewards();
	local mapTable = C_ChallengeMode.GetMapTable();
	local bestLevel = 0;
	local bestDungeon;
	for i, dungeonId in pairs(mapTable) do
		local _, weeklyBestTime, weeklyBestLevel = C_ChallengeMode.GetMapPlayerStats(dungeonId);

		if weeklyBestLevel and weeklyBestLevel > bestLevel then
			bestLevel = weeklyBestLevel;
			bestDungeon = dungeonId;
		end
	end
	
	local name, realm = UnitFullName("player")
	
	local keystone = {}
	keystone.name = name .."-" .. realm;
	keystone.class = UnitClass("player");
	keystone.isPlayers = true;
	keystone.bestLevel = bestLevel
	keystone.bestDungeon = bestDungeon
	keystone.faction = UnitFactionGroup("player") == "Alliance"
	keystone.dungeon = 0;
	keystone.level = 0;
	local newKey = false;
	for bag = 0, NUM_BAG_SLOTS do
		bagSize = GetContainerNumSlots(bag);
		if bagSize > 0 then
			for i = 1, bagSize do
				if (GetContainerItemID(bag, i) == 138019) then
					local keyParts = { strsplit(':[', GetContainerItemLink(bag, i)) }
					
					keystone.dungeon = tonumber(keyParts[2]);
					keystone.level = tonumber(keyParts[3]);
					keystone.affix1 = tonumber(keyParts[4]);
					keystone.affix2 = tonumber(keyParts[5]);
					keystone.affix3 = tonumber(keyParts[6]);
				end
			end
		end
	end
	return self:saveKeystone(keystone)
end

function KeystoneRollCall:getKeystoneIndex(characterName)
	for i, keystone in pairs(self.db.global.keystones) do
		if keystone.name == characterName then
			return i
		end
	end
	return -1
end

function KeystoneRollCall:getKeystone(characterName)
	for i, keystone in pairs(self.db.global.keystones) do
		if keystone.name == characterName then
			return keystone
		end
	end
	return nil
end

function KeystoneRollCall:getPlayerKeystoneIndex()
	local name, realm = UnitFullName("player")
	return self:getKeystoneIndex(name .."-" .. realm)
end

function KeystoneRollCall:getPlayerKeystone()
	local name, realm = UnitFullName("player")
	return self:getKeystone(name .."-" .. realm)
end